'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var TextareaAutosize = _interopDefault(require('react-autosize-textarea'));
var ResizeObserver = _interopDefault(require('resize-observer-polyfill'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var defaultState = {
  x: 0,
  y: 0,
  width: 0,
  height: 0,
  top: 0,
  right: 0,
  bottom: 0,
  left: 0
};
function useMeasure(ref, disabled) {
  if (disabled === void 0) {
    disabled = false;
  }

  var _React$useState = React.useState(defaultState),
      bounds = _React$useState[0],
      set = _React$useState[1];

  var _React$useState2 = React.useState(function () {
    return new ResizeObserver(function (_ref) {
      var entry = _ref[0];
      return set(entry.contentRect);
    });
  }),
      ro = _React$useState2[0]; // @ts-ignore: no-implicit-returns


  React.useEffect(function () {
    if (ref.current && !disabled) {
      ro.observe(ref.current);
      return function () {
        return ro.disconnect();
      };
    }
  }, [ref, ro, disabled]);
  return bounds;
}

function useOnClickOutside(
/* eslint-disable-next-line @typescript-eslint/no-explicit-any */
ref,
/* eslint-disable-next-line @typescript-eslint/no-explicit-any */
handler) {
  React.useEffect(function () {
    /* eslint-disable-next-line @typescript-eslint/no-explicit-any */
    var refs = Array.isArray(ref) ? ref : [ref];
    /* eslint-disable-next-line @typescript-eslint/no-explicit-any */

    var listener = function listener(event) {
      var isOutside = refs.reduce(function (acc, refValue) {
        if (refValue.current && refValue.current.contains(event.target)) {
          return false;
        }

        return acc;
      }, true);

      if (isOutside) {
        handler(event);
      }
    };

    document.addEventListener('mousedown', listener);
    document.addEventListener('touchstart', listener);
    return function () {
      document.removeEventListener('mousedown', listener);
      document.removeEventListener('touchstart', listener);
    };
  }, [ref, handler]);
}

function getWordAtPosition(value, startPosition, endPosition) {
  var letterIndex = 0;
  var words = value.replace(/\n/g, ' ').split(' ');

  if (typeof endPosition === 'number' && startPosition !== endPosition) {
    var word = value.substring(startPosition, endPosition);
    return [word, startPosition, endPosition];
  }

  for (var i = 0; i < words.length; i++) {
    var _word = words[i];
    var startIndex = letterIndex;
    letterIndex += _word.length;

    if (letterIndex >= startPosition) {
      return [_word.replace(/\r?\n|\r/g, '').trim(), startIndex, startIndex + _word.length];
    }

    letterIndex += 1;
  }

  return ['', 0, 0];
}
function getRows(value, position) {
  var letterIndex = 0;
  var rows = value.split(/\n/g);
  var currentIndex = 0;

  for (var i = 0; i < rows.length; i++) {
    var row = rows[i];
    letterIndex += row.length;

    if (letterIndex >= position) {
      currentIndex = i;
    }

    letterIndex += 1;
  }

  return {
    rows: rows,
    currentIndex: currentIndex
  };
}
function getRowAtPosition(value, position) {
  var _getRows = getRows(value, position),
      rows = _getRows.rows,
      currentIndex = _getRows.currentIndex;

  return rows[currentIndex];
}
function isCtrlCmd(event) {
  var isMacLike = /(Mac|iPhone|iPod|iPad)/i.test(navigator.platform);
  return isMacLike && event.metaKey || !isMacLike && event.ctrlKey;
}
function toggleWordWrap(el, value) {
  var caretStartingPosition = el.selectionStart;
  var caretEndingPosition = el.selectionEnd;
  var hasSelection = caretStartingPosition !== caretEndingPosition;

  var _getWordAtPosition = getWordAtPosition(el.value, caretStartingPosition, caretEndingPosition),
      word = _getWordAtPosition[0],
      startPosition = _getWordAtPosition[1],
      endPosition = _getWordAtPosition[2];

  var firstChars = word.substr(0, value.length);
  var lastChars = word.substr(-value.length);
  var beforeWord = el.value.substr(0, startPosition);
  var afterWord = el.value.substr(endPosition, el.value.length);

  if (firstChars !== value || lastChars !== value) {
    el.value = "" + beforeWord + value + word + value + afterWord;

    if (hasSelection) {
      el.setSelectionRange(caretStartingPosition, caretEndingPosition + value.length * 2);
    } else {
      el.setSelectionRange(caretStartingPosition + value.length, caretEndingPosition + value.length);
    }
  } else {
    var updatedWord = word.substr(value.length, word.length - value.length * 2);
    el.value = "" + beforeWord + updatedWord + afterWord;

    if (hasSelection) {
      el.setSelectionRange(caretStartingPosition, caretEndingPosition - value.length * 2);
    } else {
      el.setSelectionRange(caretStartingPosition - value.length, caretEndingPosition - value.length);
    }
  }

  return el;
}

function getEditorData(el, insertStart, insertEnd) {
  var caretStartingPosition = el.selectionStart;
  var caretEndingPosition = el.selectionEnd;
  var hasSelected = caretStartingPosition !== caretEndingPosition;
  var value = el.value;
  var caretStart = el.selectionStart;
  var caretEnd = el.selectionEnd;
  var valueUpToStart = value.substr(0, caretStart);

  if (insertStart) {
    var valueAfter = value.substr(caretStart, value.length);
    caretStart = (valueUpToStart + insertStart).length;
    caretEnd = caretEnd + insertStart.length;
    value = valueUpToStart + insertStart + valueAfter;
    el.value = value;
  }

  if (insertEnd) {
    var _valueAfter = value.substr(caretEnd, value.length);

    valueUpToStart = value.substr(0, caretEnd);
    caretEnd = (valueUpToStart + insertEnd).length;
    value = valueUpToStart + insertEnd + _valueAfter;
    el.value = value;
  }

  if (insertStart || insertEnd) {
    if (hasSelected) {
      caretStart = caretStart - (insertStart || '').length;
      caretEnd = caretEnd + (insertEnd || '').length;
      el.setSelectionRange(caretStart, caretEnd);
    } else {
      el.setSelectionRange(caretStart, caretStart);
    }
  }

  var _getWordAtPosition = getWordAtPosition(value, caretStart),
      currentWord = _getWordAtPosition[0];

  return {
    value: value,
    currentWord: currentWord,
    valueUpToStart: valueUpToStart,
    lastSelectionStart: caretStart
  };
}

function reducer(state, action) {
  switch (action.type) {
    case 'EditorData':
      return _extends({}, state, {
        editor: _extends({}, getEditorData(action.payload))
      });

    case 'EditorSize':
      return _extends({}, state, {}, action.payload);

    case 'AddTab':
      var tabValue = '  ';
      return _extends({}, state, {
        editor: _extends({}, getEditorData(action.payload, tabValue))
      });

    case 'AddListItem':
      var listValue = '\n- ';

      if (action.payload.type === 'ol') {
        var row = getRowAtPosition(action.payload.el.value, action.payload.el.selectionStart);
        var listIndex = parseFloat(row.substr(0, 3));

        if (listIndex === 1) {
          listValue = '\n1. ';
        } else {
          listValue = "\n" + (listIndex + 1) + ". ";
        }
      }

      return _extends({}, state, {
        editor: _extends({}, getEditorData(action.payload.el, listValue))
      });

    case 'ClearList':
      var _getRows = getRows(action.payload.value, action.payload.selectionStart),
          rows = _getRows.rows,
          currentIndex = _getRows.currentIndex;

      var updatedRows = [].concat(rows);
      updatedRows[currentIndex] = '';
      action.payload.value = updatedRows.join('\n');
      return _extends({}, state, {
        editor: _extends({}, getEditorData(action.payload))
      });

    case 'ToggleBold':
      return _extends({}, state, {
        editor: _extends({}, getEditorData(toggleWordWrap(action.payload, '**')))
      });

    case 'ToggleItalic':
      return _extends({}, state, {
        editor: _extends({}, getEditorData(toggleWordWrap(action.payload, '_')))
      });

    case 'Enclose':
      var _getWordAtPosition2 = getWordAtPosition(action.payload.el.value, action.payload.el.selectionStart, action.payload.el.selectionEnd),
          currentWord = _getWordAtPosition2[0],
          currentWordEnd = _getWordAtPosition2[2];

      var startChar = action.payload.type;
      var endChar = startChar === '[' ? ']' : startChar === '(' ? ')' : '}';

      if (action.payload.el.selectionStart !== action.payload.el.selectionEnd || currentWord === startChar) {
        return _extends({}, state, {
          editor: _extends({}, getEditorData(action.payload.el, startChar, endChar))
        });
      } else if (currentWordEnd === action.payload.el.selectionEnd) {
        return _extends({}, state, {
          editor: _extends({}, getEditorData(action.payload.el, startChar, endChar))
        });
      }

      return _extends({}, state, {
        editor: _extends({}, getEditorData(action.payload.el, action.payload.type))
      });

    default:
      return state;
  }
}

var initialState = {
  editor: {
    currentWord: '',
    value: '',
    valueUpToStart: '',
    lastSelectionStart: 0
  },
  width: 0,
  height: 0
};
var EditorContet =
/*#__PURE__*/
React.createContext(undefined);
var Editor = function Editor(_ref) {
  var defaultValue = _ref.defaultValue,
      onChange = _ref.onChange,
      onSubmit = _ref.onSubmit,
      onBlur = _ref.onBlur,
      onCancel = _ref.onCancel,
      disableFormatting = _ref.disableFormatting,
      singleLine = _ref.singleLine,
      children = _ref.children,
      rest = _objectWithoutPropertiesLoose(_ref, ["defaultValue", "onChange", "onSubmit", "onBlur", "onCancel", "disableFormatting", "singleLine", "children"]);

  var containerRef = React.useRef(null);
  var editorRef = React.useRef(null);

  var _React$useReducer = React.useReducer(reducer, initialState, function (initial) {
    return _extends({}, initial, {
      editor: _extends({}, initial.editor, {
        value: defaultValue || ''
      })
    });
  }),
      state = _React$useReducer[0],
      dispatch = _React$useReducer[1];

  var _useMeasure = useMeasure(editorRef),
      width = _useMeasure.width,
      height = _useMeasure.height;

  useOnClickOutside(containerRef, function () {
    if (onBlur) {
      onBlur();
    }
  });
  React.useEffect(function () {
    dispatch({
      type: 'EditorSize',
      payload: {
        width: width,
        height: height
      }
    });
  }, [width, height]); // @ts-ignore noImplicitReturns

  React.useEffect(function () {
    var editor = editorRef.current;

    if (editor) {
      var handleKeyEvent = function handleKeyEvent(event) {
        var CtrlCmd = isCtrlCmd(event);

        if (CtrlCmd && event.key === 'Enter' && onSubmit) {
          onSubmit();
        }

        if (event.key === 'Escape' && onCancel) {
          onCancel();
        }

        if (event.key === 'Enter') {
          if (!disableFormatting) {
            var row = getRowAtPosition(editor.value, editor.selectionStart);
            var rowStartingChar = row.substr(0, 2);
            var olRegex = new RegExp(/^[0-9]. /);
            var isUL = rowStartingChar === '- ';
            var isOL = olRegex.test(row);

            if (isUL || isOL) {
              if (isUL && row === rowStartingChar || isOL && row.substr(0, 3) === row) {
                dispatch({
                  type: 'ClearList',
                  payload: editor
                });
                return;
              }

              dispatch({
                type: 'AddListItem',
                payload: {
                  el: editor,
                  type: isUL ? 'ul' : 'ol'
                }
              });
              event.preventDefault();
            }
          }

          if (singleLine) {
            event.preventDefault();
          }
        }

        if (!disableFormatting) {
          if (event.key === 'Tab') {
            event.preventDefault();
            dispatch({
              type: 'AddTab',
              payload: editor
            });
          }

          if (CtrlCmd && event.key === 'b') {
            dispatch({
              type: 'ToggleBold',
              payload: editor
            });
          }

          if (CtrlCmd && event.key === 'i') {
            dispatch({
              type: 'ToggleItalic',
              payload: editor
            });
          }

          if (['[', '(', '{'].includes(event.key)) {
            dispatch({
              type: 'Enclose',
              payload: {
                el: editor,
                type: event.key
              }
            });
            event.preventDefault();
          }

          if ([']', ')', '}'].includes(event.key)) {
            var noSelection = editor.selectionStart === editor.selectionEnd;
            var currentPosition = editor.selectionStart;
            var nextCharacater = editor.value[editor.selectionStart];

            if (noSelection && nextCharacater === event.key) {
              editor.setSelectionRange(currentPosition + 1, currentPosition + 1);
              event.preventDefault();
            }
          }
        }
      };

      editor.addEventListener('keydown', handleKeyEvent, false);
      return function () {
        editor.removeEventListener('keydown', handleKeyEvent, false);
      };
    }
  }, [onSubmit, onCancel, onBlur, disableFormatting, singleLine]);

  var handleChange = function handleChange(event) {
    dispatch({
      type: 'EditorData',
      payload: event.target
    });
  };

  React.useEffect(function () {
    if (onChange) {
      onChange(state.editor.value);
    }
  }, [onChange, state.editor.value]);
  return React.createElement(EditorContet.Provider, {
    value: {
      state: state,
      dispatch: dispatch,
      editorRef: editorRef
    }
  }, React.createElement("div", {
    "data-testid": "container",
    ref: containerRef,
    className: "rmm-container"
  }, React.createElement(TextareaAutosize, Object.assign({}, rest, {
    "data-testid": "textarea",
    ref: editorRef,
    className: "rmm-editor",
    wrap: "hard",
    onChange: handleChange,
    value: state.editor.value
  })), children));
};

function useEditor() {
  var context = React.useContext(EditorContet);

  if (!context) {
    throw new Error('Component has to be wrapped inside <Editor />');
  }

  return context;
}

function fuzzySearch(needle, haystack) {
  if (needle.length > haystack.length) {
    return false;
  }

  outer: for (var i = 0, j = 0; i < needle.length; i++) {
    var nch = needle.charCodeAt(i);

    while (j < haystack.length) {
      if (haystack.charCodeAt(j++) === nch) {
        continue outer;
      }
    }

    return false;
  }

  return true;
}

var Mention = function Mention(_ref) {
  var prefix = _ref.prefix,
      data = _ref.data,
      onSearch = _ref.onSearch,
      maxItems = _ref.maxItems;

  var _useEditor = useEditor(),
      state = _useEditor.state,
      dispatch = _useEditor.dispatch,
      editorRef = _useEditor.editorRef;

  var _state$editor = state.editor,
      currentWord = _state$editor.currentWord,
      valueUpToStart = _state$editor.valueUpToStart,
      value = _state$editor.value;
  var width = state.width;
  var show = currentWord[0] === prefix;

  var _React$useState = React.useState([0, 0]),
      _React$useState$ = _React$useState[0],
      x = _React$useState$[0],
      y = _React$useState$[1],
      setPosition = _React$useState[1];

  var _React$useState2 = React.useState(0),
      selectedIndex = _React$useState2[0],
      setSelectedIndex = _React$useState2[1];

  var withoutPrefix = currentWord.replace(prefix, '');

  var _React$useState3 = React.useState(false),
      forceHide = _React$useState3[0],
      setForceHide = _React$useState3[1];

  var filtered = data.filter(function (item) {
    return fuzzySearch(withoutPrefix, "" + item.value + item.label);
  }).slice(0, maxItems);
  React.useEffect(function () {
    setForceHide(false);
  }, [withoutPrefix]);
  React.useEffect(function () {
    if (show && onSearch) {
      onSearch(withoutPrefix);
    }
  }, [show, withoutPrefix, onSearch]); // @ts-ignore noImplicitReturns

  React.useEffect(function () {
    var editor = editorRef.current;

    if (show && editor) {
      var div = document.createElement('div');
      var indicatorData = valueUpToStart.split(/\n\r?/g);
      var span = document.createElement('span');
      div.classList.add('rmm-editor');
      div.style.cssText = "max-width: " + width + "px; position: absolute; top: 0; left: -9999px; opacity: 0; color: transparent; pointer-events: none; white-space: pre-wrap;";
      span.innerHTML = '&nbsp;';
      span.style.position = 'absolute';
      div.innerHTML = indicatorData.join('<br />');
      div.append(span);
      document.body.appendChild(div);
      setPosition([span.offsetLeft, span.offsetTop + span.clientHeight + 2]);
      return function () {
        return document.body.removeChild(div);
      };
    }
  }, [show, width, editorRef, valueUpToStart]);
  var insertValue = React.useCallback(function (node, selectedValue) {
    var valueUpToValue = value.slice(0, node.selectionStart - currentWord.length) + ("" + prefix + selectedValue.value + " ");
    var updatedValue = valueUpToValue + value.slice(node.selectionStart, value.length);
    var start = valueUpToValue.length;
    var end = valueUpToValue.length;
    node.value = updatedValue;
    node.setSelectionRange(start, end);
    dispatch({
      type: 'EditorData',
      payload: node
    });
    setSelectedIndex(0);
  }, [currentWord.length, dispatch, prefix, value]); // @ts-ignore noImplicitReturns

  React.useEffect(function () {
    var editor = editorRef.current;

    if (show) {
      if (selectedIndex > filtered.length) {
        setSelectedIndex(0);
      }

      var handleKeyDown = function handleKeyDown(event) {
        if (event.key === 'Escape') {
          setForceHide(true);
        }

        if (event.key === 'Enter') {
          var selectedValue = filtered[selectedIndex] || filtered[0];

          if (!selectedValue) {
            return;
          }

          var node = event.target;
          insertValue(node, selectedValue);
          event.stopPropagation();
          event.preventDefault();
        }

        if (event.key === 'ArrowUp') {
          setSelectedIndex(function (current) {
            return Math.abs((current - 1) % filtered.length);
          });
          event.preventDefault();
        }

        if (event.key === 'ArrowDown') {
          setSelectedIndex(function (current) {
            return Math.abs((current + 1) % filtered.length);
          });
          event.preventDefault();
        }
      };

      if (editor) {
        editor.addEventListener('keydown', handleKeyDown, false);
        return function () {
          editor.removeEventListener('keydown', handleKeyDown, false);
        };
      }
    }
  }, [show, currentWord, value, prefix, filtered, selectedIndex, dispatch, editorRef, insertValue]);

  if (!show || !editorRef.current || forceHide) {
    return null;
  }

  var handleItemClick = function handleItemClick(data) {
    return function () {
      if (editorRef.current) {
        insertValue(editorRef.current, data);
        editorRef.current.focus();
      }
    };
  };

  return React.createElement("div", {
    className: "rmm-list",
    style: {
      transform: "translate(" + x + "px, " + (y - editorRef.current.scrollTop) + "px)"
    }
  }, filtered.map(function (item, index) {
    return React.createElement("div", {
      key: item.value,
      role: "button",
      className: "rmm-list-item " + (index === selectedIndex && 'rmm-list-item-selected'),
      onMouseOver: function onMouseOver() {
        return setSelectedIndex(index);
      },
      onClick: handleItemClick(item)
    }, React.createElement("b", null, item.value), " ", item.label);
  }));
};

exports.Editor = Editor;
exports.EditorContet = EditorContet;
exports.Mention = Mention;
//# sourceMappingURL=react-marky-markdown.cjs.development.js.map
